#!perl -w

use strict;
use Test::More tests => 6;

use_ok('Net::Gnip::Activity');
use_ok('Net::Gnip::ActivityStream');
use_ok('Net::Gnip::Filter');
use_ok('Net::Gnip::FilterStream');
use_ok('Net::Gnip::Publisher');
use_ok('Net::Gnip::PublisherStream');
